#include "ProgramManager.h"
#include <utils.h>
#include <SimplexNoise1234.h>
#include <NoiseSlice.h>

ProgramManager* ProgramManager::m_Manager = NULL;

ProgramManager::ProgramManager(void)
{
	m_Manager = this;
	m_MouseState = MOUSE_INACTIVE;
	mousedown_x = 0;
	mousedown_y = 0;
	should_update_noise = false;
	m_noiseSlice.InitializeNoiseBitmap(WINDOW_WIDTH, WINDOW_HEIGHT);

}

void ProgramManager::UpdateTexture()
{
	//creating 2d bitmap texture.
	
	// Create The Texture
	glGenTextures(1, &m_noiseTexture[0]);
	glBindTexture(GL_TEXTURE_2D, m_noiseTexture[0]);
//	glPixelStorei(GL_UNPACK_ALIGNMENT, 3);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear Min Filter
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear Mag Filter

	if(!m_Manager->m_noiseSlice.m_bitmapBits)
	{
		GLubyte gray = 0x33;
		glTexImage2D(GL_TEXTURE_2D, 0, 1, 1, 1, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, &gray);
	}
	else
		glTexImage2D(GL_TEXTURE_2D, 0, 3, m_Manager->m_noiseSlice.m_bitmapWidth, m_Manager->m_noiseSlice.m_bitmapHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, m_Manager->m_noiseSlice.m_bitmapBits);
}

ProgramManager::~ProgramManager(void)
{
	m_Manager = NULL;
}

void *font = GLUT_BITMAP_TIMES_ROMAN_24;
void *fonts[] =
{
  GLUT_BITMAP_9_BY_15,
  GLUT_BITMAP_TIMES_ROMAN_10,
  GLUT_BITMAP_TIMES_ROMAN_24
};
char defaultMessage[] = "GLUT means OpenGL.";
char *message = defaultMessage;


static DWORD WINAPI DrawBufferThreadProc(__in  LPVOID lpParameter);

void ProgramManager::selectFont(int newfont)
{
  font = fonts[newfont];
  glutPostRedisplay();
}

void ProgramManager::menu_NoiseControl(int value)
{
	switch(value)
	{
	case 1:
		m_Manager->m_noiseSlice.ResetCoords();
		m_Manager->should_update_noise = true;
		break;
	}
}

void ProgramManager::mousebutton(int button, int state, int x, int y)
{
	if(button == GLUT_LEFT_BUTTON)
	{
		if(state == GLUT_DOWN)
		{
			if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
				m_Manager->m_MouseState = MOUSE_CUTOFF;
			else if(glutGetModifiers() & GLUT_ACTIVE_CTRL)
				m_Manager->m_MouseState = MOUSE_ZOOMING;
			else
				m_Manager->m_MouseState = MOUSE_PANNING;

			m_Manager->mousedown_x = x;
			m_Manager->mousedown_y = y;
		}
		else
			m_Manager->m_MouseState = MOUSE_INACTIVE;
	}
}

void ProgramManager::mousemove(int x, int y)
{
	if(m_Manager->m_MouseState != MOUSE_INACTIVE)
	{
		int delta_x = m_Manager->mousedown_x - x;
		int delta_y = m_Manager->mousedown_y - y;
		m_Manager->mousedown_x = x;
		m_Manager->mousedown_y = y;

		switch(m_Manager->m_MouseState)
		{
		case MOUSE_PANNING:
			m_Manager->m_noiseSlice.MoveCoordinates(delta_x, delta_y);
			break;
		case MOUSE_ZOOMING:
			m_Manager->m_noiseSlice.IncrementZoom(((float)delta_y)/10);
			break;
		case MOUSE_CUTOFF:
			m_Manager->m_noiseSlice.IncrementCuttoff(((float)delta_y)/100);
			break;
		}
		
		m_Manager->should_update_noise = true;
	}
}

void ProgramManager::tick(int value)
{
	if(m_Manager->should_update_noise)
	{
		m_Manager->should_update_noise = false;
		m_Manager->m_noiseSlice.WriteNoiseBitmap();
		m_Manager->UpdateTexture();
		glutPostRedisplay();
	}

	glutTimerFunc(10, ProgramManager::tick, 0);
}

void ProgramManager::output(int x, int y, char *string)
{
  int len, i;

  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(font, string[i]);
  }
}
void ProgramManager::display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();									// Reset The View

	glBindTexture(GL_TEXTURE_2D, m_Manager->m_noiseTexture[0]);

	glBegin(GL_QUADS);
		// Front Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(0,		0,  0);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(1.0f,	0.0f,  0);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(1.0f,  1.0f,  0);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(0.0f,  1.0f,  0);
	glEnd();

	glutSwapBuffers();
}

void ProgramManager::reshape(int w, int h)
{
	glViewport(0, 0, w, h);
	m_Manager->m_textureWidth = w;
	m_Manager->m_textureHeight = h;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, -1, 1);
	//gluOrtho2D(0, w, h, 0, -10, 10);
	glMatrixMode(GL_MODELVIEW);
	glEnable(GL_TEXTURE_2D);
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);

}

int ProgramManager::Run(int argc, char **argv)
{
  int i;//, msg_submenu, color_submenu;
  glutInit(&argc, argv);
  for (i = 1; i < argc; i++) {
    if (!strcmp(argv[i], "-mono")) {
      font = GLUT_BITMAP_9_BY_15;
    }
  }

  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
  glutCreateWindow("NoiseViewer v1.0");

  glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping ( NEW )
  glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
  glClearDepth(1.0f);									// Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
  glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations
  
  glutDisplayFunc(ProgramManager::display);
  glutReshapeFunc(ProgramManager::reshape);
  glutTimerFunc(10, ProgramManager::tick, 0);
  glutMouseFunc(ProgramManager::mousebutton);
  glutMotionFunc(ProgramManager::mousemove);
  m_textureWidth = WINDOW_WIDTH;
  m_textureHeight = WINDOW_HEIGHT;
  should_update_noise = true;

  //glutAddMenuEntry("abc", 1);
  //glutAddMenuEntry("ABC", 2);
  //color_submenu = glutCreateMenu(selectColor);
  //glutAddMenuEntry("Green", 1);
  //glutAddMenuEntry("Red", 2);
  //glutAddMenuEntry("White", 3);
  //glutCreateMenu(selectFont);
  //glutAddMenuEntry("9 by 15", 0);
  //glutAddMenuEntry("Times Roman 10", 1);
  //glutAddMenuEntry("Times Roman 24", 2);
  //glutAddSubMenu("Messages", msg_submenu);
  //glutAddSubMenu("Color", color_submenu);
  int submenu_noise = glutCreateMenu(ProgramManager::menu_NoiseControl);
//  //glutAddSubMenu("Noise controls", submenu_noise);
  glutAddMenuEntry("Reset noise", 1);

  glutAttachMenu(GLUT_RIGHT_BUTTON);

  glutMainLoop();
  return 0;
}