#include <NoiseSlice.h>


NoiseSlice::NoiseSlice()
{
	m_bitmapBits = NULL;
	m_bitmapWidth = 0;
	m_bitmapHeight = 0;

	ResetCoords();
}

//resets all logical coordinates, zoom, and cutoff.
void NoiseSlice::ResetCoords()
{
	m_cutoff = 0.5;
	m_coords.x = 0;
	m_coords.y = 0;
	m_zoomx = 20;
	m_zoomy = 20;
	UpdateWindowMapper();
}

void NoiseSlice::InitializeNoiseBitmap(int width, int height)
{
	m_bitmapWidth = width;
	m_bitmapHeight = height;

	delete[] m_bitmapBits;
	m_bitmapBits = NULL;

	m_bitmapBits = new GLubyte[3*m_bitmapWidth*m_bitmapHeight];

	//memset(m_bitmapBits, 0, 3*m_bitmapWidth*m_bitmapHeight);

	UpdateWindowMapper();
}

void NoiseSlice::MoveCoordinates(int delta_x, int delta_y)
{
	//xdiff, ydiff are in PIXELS
	//we want to translate to noisespace.

	//REIMPLEMENT
	float xdiff = delta_x, ydiff = delta_y;
	m_windowMapping.PixelDeltaToNoiseDelta(xdiff, ydiff);
	m_coords.x += xdiff;
	m_coords.y += ydiff;
	UpdateWindowMapper();
}

void NoiseSlice::ChangeZoom(float zoom)
{
	zoom = MAX(0.0001, zoom);
	m_zoomx = m_zoomy = zoom;
	UpdateWindowMapper();
}

void NoiseSlice::IncrementZoom(float deltazoom)
{
	ChangeZoom(m_zoomx + deltazoom);
}

void NoiseSlice::SetCoordinates(float x, float y, float zoomX, float zoomY)
{
	assert(zoomX > 0);
	assert(zoomY > 0);
	m_zoomy = zoomY;
	m_zoomx = zoomX;
	m_coords.x = x;
	m_coords.y = y;

	UpdateWindowMapper();
}

//buffer is >= 100 chars
//void NoiseSlice::PrintCoordinates(WCHAR* text)
//{
//	_snwprintf_s(text, 100, 100, L"X: %g\nY: %g\nZoom X: %g\nZoom Y: %g", m_coords.x, m_coords.y, m_zoomx, m_zoomy);
//}

NoiseSlice::~NoiseSlice(void)
{
	delete [] m_bitmapBits;
}


void NoiseSlice::ChangeCutoff(float newCutoff)
{
	m_cutoff = MAX(0.0f, newCutoff);
	m_cutoff = MIN(1.0f, newCutoff);
	LOG(TEXT("doing cutoff: %f"), newCutoff);
}

void NoiseSlice::IncrementCuttoff(float deltacutoff)
{
	ChangeCutoff(m_cutoff + deltacutoff);
}

void NoiseSlice::UpdateWindowMapper()
{
	if(m_bitmapWidth > 0 && m_bitmapHeight > 0)
	{
		m_windowMapping.SetNoiseCoordinates(m_coords.y - m_zoomy/2, m_coords.x + m_zoomx/2, m_coords.x - m_zoomx/2, m_coords.y + m_zoomy/2);
		m_windowMapping.SetDestinationWindow(0, m_bitmapWidth, 0, m_bitmapHeight);
	}
}



void NoiseSlice::WriteNoiseBitmap()
{
	GLubyte* dest = m_bitmapBits;

	GLubyte gray;

	for(int i = 0; i < m_bitmapWidth; i++)
	{
		for(int j = 0; j < m_bitmapHeight; j++)
		{
			float xVal = j, yVal = i;
			m_windowMapping.PixelPositionToNoiseSpace(xVal, yVal);

			float noise = SimplexNoise1234::noise( xVal, yVal );

			float res = (noise + 1.0f)*0.5f;
//			if (res > m_DBG_
			if(res > m_cutoff)
			{
				float scalefactor = (res - m_cutoff) * (1/(1 - m_cutoff));
				scalefactor = MIN(scalefactor, 1.0f);
				scalefactor = MAX(scalefactor, 0.0f);
				gray = (GLubyte)(255 * scalefactor);
			}
			else
				gray = 0;
			//BYTE gray = 100;
			//if(i < 100 && j < 100)
			//	gray = 255;
			*(dest++) = gray;
			*(dest++) = gray;
			*(dest++) = gray;
			//DWORD final = gray | gray<<8 | gray << 16 | gray<<24;
			//bitmapBits[i*WINDOW_WIDTH + j] = final;
		}
	}
}