#include <WindowMapping.h>
#include <stdio.h>

WindowMapping::WindowMapping()
{
	noiseSpaceWindow.bottom = noiseSpaceWindow.left = noiseSpaceWindow.right = noiseSpaceWindow.top = 0.0f;
	bitmapWindow.bottom = bitmapWindow.left = bitmapWindow.right = bitmapWindow.top = 0.0f;
	m_bitmapWidth = m_bitmapHeight = m_noiseWindowWidth = m_noiseWindowHeight = 0.0f;
}

WindowMapping::~WindowMapping()
{
}

void WindowMapping::SetNoiseCoordinates(float top, float right, float left, float bottom)
{
	assert(right > left);
	assert(bottom > top);

	noiseSpaceWindow.top = top;
	noiseSpaceWindow.bottom = bottom;
	noiseSpaceWindow.right = right;
	noiseSpaceWindow.left = left;

	m_noiseWindowWidth = right - left;
	m_noiseWindowHeight = top - bottom;
}

void WindowMapping::SetDestinationWindow(float top, float right, float left, float bottom)
{
	assert(right > left);
	assert(bottom > top);

	bitmapWindow.top = top;
	bitmapWindow.bottom = bottom;
	bitmapWindow.right = right;
	bitmapWindow.left = left;

	m_bitmapWidth = right - left;
	m_bitmapHeight = top - bottom;
}

void WindowMapping::PixelPositionToNoiseSpace(float & x, float & y)
{
	float fractionX = (x - bitmapWindow.left) / (m_bitmapWidth);
	float fractionY = (y - bitmapWindow.top) / (m_bitmapHeight);

	x = noiseSpaceWindow.left + m_noiseWindowWidth*fractionX;
	y = noiseSpaceWindow.top + m_noiseWindowHeight*fractionY;
}

void WindowMapping::GetPixelSizeInNoiseSpace(float& xpixelsize, float& ypixelsize)
{
	float x = 1.0, y = 1.0;
	float fractionX = x / ((float)WINDOW_WIDTH);
	float fractionY = y / ((float)WINDOW_HEIGHT);

	xpixelsize = m_noiseWindowWidth*fractionX;
	ypixelsize = m_noiseWindowHeight*fractionY;
	//LOG(L"Pixel size in noise space: (%g, %g)", xpixelsize, ypixelsize);
}


void WindowMapping::PixelDeltaToNoiseDelta(float & xdiff, float & ydiff)
{
	float xpixel, ypixel;
	GetPixelSizeInNoiseSpace(xpixel, ypixel);
	xdiff = xdiff * xpixel;
	ydiff = ydiff * ypixel;
}